package com.marvinelsen.willow.ui.cells

import com.marvinelsen.willow.config.Config
import com.marvinelsen.willow.config.Script
import com.marvinelsen.willow.ui.SentenceFx
import javafx.beans.binding.Bindings
import javafx.scene.control.Label
import javafx.scene.control.ListCell
import javafx.scene.control.ListView
import javafx.scene.layout.VBox
import javafx.util.Callback

class SentenceCellFactory(private val config: Config) : Callback<ListView<SentenceFx?>, ListCell<SentenceFx?>> {

    override fun call(listView: ListView<SentenceFx?>): ListCell<SentenceFx?> {
        val sentenceCell = SentenceCell(config)
        sentenceCell.prefWidthProperty().bind(listView.widthProperty().subtract(CELL_PADDING))
        return sentenceCell
    }

    companion object {
        private const val CELL_PADDING = 16
    }
}

private class SentenceCell(private val config: Config) : ListCell<SentenceFx?>() {
    private val labelSentence = Label().apply {
        styleClass.add("sentence")
    }
    private val root = VBox(labelSentence)

    init {
        text = null
    }

    override fun updateItem(sentence: SentenceFx?, empty: Boolean) {
        super.updateItem(sentence, empty)
        if (empty || sentence == null) {
            graphic = null
        } else {
            labelSentence.textProperty().bind(
                Bindings.createStringBinding(
                    {
                        when (config.script.value!!) {
                            Script.SIMPLIFIED -> sentence.simplifiedProperty.value
                            Script.TRADITIONAL -> sentence.traditionalProperty.value
                        }
                    },
                    config.script
                )
            )

            graphic = root
        }
    }
}
